#include "scalar.hpp"

std::string numeric_traits<large_real>::to_string(const large_real& v, const interpreter_context& context) {
    mpfr::real_exp_t dot_pos, exp;
    char* ch = mpfr_get_str(
        0, &dot_pos, 10, context.digits, v._x, MPFR_REAL_CLASS_RND_DFLT
    );

    if (!ch) return "";

    std::string s = ch;
    mpfr_free_str(ch);

    // The exponent is the dot position minus one.
    // Decrement it and check for overflow.
    exp = dot_pos;
    if (dot_pos < --exp) return "0";

    bool negative = false;
    if (s[0] == '-') {
        negative = true;
        s.erase(0, 1);
    }

    if (exp < 0) {
        if (exp > -5) {
            s.insert(0, "0.");

            for (size_t i = 0; i < size_t(-1-exp); ++i)
                s.insert(2, "0");

            while (s[s.size()-1] == '0')
                s.erase(s.size()-1, 1);

            if (s[s.size()-1] == '.')
                s.erase(s.size()-1, 1);

            return negative ? "-" + s : s;
        }
    } else {
        if (size_t(exp) < context.digits) {
            if (size_t(exp) != context.digits)
                s.insert(dot_pos, ".");

            while (s[s.size()-1] == '0')
                s.erase(s.size()-1, 1);

            if (s[s.size()-1] == '.')
                s.erase(s.size()-1, 1);

            return negative ? "-" + s : s;
        }
    }

    s.insert(1, ".");

    while (s[s.size()-1] == '0')
        s.erase(s.size()-1, 1);

    if (s[s.size()-1] == '.')
        s.erase(s.size()-1, 1);

    std::stringstream ss;
    ss << exp;
    s += "e" + (exp > 0 ? "+" + ss.str() : ss.str());

    return negative ? "-" + s : s;
}
